module AttributeCache
  class AttributeCache
    def initialize(name:, attribute:, callback:)
      @name = name
      @attribute = attribute
      @callback = callback
    end

    def set_attribute(record, save: false, args: [])
      record.write_attribute(@attribute, execute_callback(record, *args))
      record.save if save
    end
    alias invalidate_attribute set_attribute

    def set_attribute!(record, args: [])
      set_attribute(record, save: true, args: args)
    end
    alias invalidate_attribute! set_attribute!

    def value(record, save:, keep_blanks:, args: [])
      if cached_value(record).nil? || !keep_blanks && cached_value(record).blank?
        set_attribute(record, save: save, args: args)
      end
      cached_value(record)
    end

    private

    def cached_value(record)
      record.read_attribute(@attribute)
    end

    def execute_callback(record, *args)
      case @callback
      when Symbol, String
        record.send(@callback, *args)
      when Proc
        @callback[*args]
      else
        raise 'An invalid callback was provided!'
      end
    end
  end
end